# bzutils -- Python module to interact with bugzilla servers.
# Copyright (C) 2007  Gustavo R. Montesino
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import cookielib
import csv
import urllib
import urllib2
import re

import bchart
from bug import Bug
from exceptions import *

class CGI:
    """Access bugzilla servers through cgi/parsing"""

    def __init__(self, baseurl, username=None, password=None):
        
        self.baseurl = baseurl.rstrip("/")
        self.usearname = username
        self.password = password

        # Bugzilla's cookies setup
        self.cookies = cookielib.CookieJar()
        self.urlopener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookies))

        # Set columns to return on query
        columndata = "column_bug_severity=on"
        columndata = columndata + "&column_priority=on"
        columndata = columndata + "&column_assigned_to=on"
        columndata = columndata + "&column_reporter=on"
        columndata = columndata + "&column_bug_status=on"
        columndata = columndata + "&column_resolution=on"
        columndata = columndata + "&column_product=on"
        columndata = columndata + "&column_component=on"
        columndata = columndata + "&column_short_desc=on"

        url = "%s/colchange.cgi?rememberedquery=&%s" % (self.baseurl, columndata)
        self.urlopener.open(url)
    
    def parse_csv(self, file):
        """Parses a bugzilla-generated csv file

        This function gets a bugzilla-generated csv file and returns a list
        of bug objects."""

        data = csv.reader(file)
        data.next() # skips the header

        bugs = []

        for report in data: 
            bug = Bug(report[0])
            bug.set_severity(report[1])
            bug.set_priority(report[2])
            bug.set_assignee(report[3])
            bug.set_reporter(report[4])
            bug.set_status(report[5])
            bug.set_resolution(report[6])
            bug.set_product(report[7])
            bug.set_component(report[8])
            bug.set_summary(report[9])
            bugs.append(bug)

        return bugs

    def query_boogle(self, querystr):
        """Query the bugzilla server using gnome's bugzilla boogle"""

        url = "%s/buglist.cgi?query=%s&ctype=csv" % (self.baseurl.rstrip("/"), querystr)
        f = self.urlopener.open(url)

        return self.parse_csv(f)

    def query_bchart(self, charts=None, str=None):
        """Query the bugzilla server through boolean charts

        This function queries bugzilla through boolean charts.
        The boolean charts to use must be passed as a chart
        list or as a string representation, see the functions
        to_url and str_to_charts on bchart.py for the syntax.
        
        This function will return a list of bugs which match the
        conditions of all charts."""

        if str:
            charts = bchart.str_to_chart(str)

        url = self.baseurl.rstrip("/") + "/buglist.cgi?query_format=advanced&ctype=csv"
        url = url + "&" + urllib.urlencode(bchart.to_url(charts))

        f = self.urlopener.open(url)
        return self.parse_csv(f)

# vim: tabstop=4 expandtab shiftwidth=4
